/*
 * Arduino Workbench tekyinblack 2020 - www.tekyinblack.com
 */
/*
 * Arduino Workbench
 * credits to ADAFruit for their GFX library and Touchscreen code
 *            SwitchDoc Labs for their INA3221 library
 *            David Prentice for his MCUFRIEND_kbv library
 * 
 * V01 first formal version
 * 
 * 
 * // logic functions interpret main code requirements for sepcific hardware implementations
 * // if using a different board or library these functions may have to be updated
 */
// the regular Adafruit "TouchScreen.h" library only works on AVRs

// different mcufriend shields have Touchscreen on different pins
// and rotation.
// Run the TouchScreen_Calibr_native sketch for calibration of your shield

#include <MCUFRIEND_kbv.h>
MCUFRIEND_kbv tft;       
#include <TouchScreen.h>

// touch screen may require calibration before use to obtain these values
// const int XP=8,XM=A2,YP=A3,YM=9; //ID=0x9341
const int XP=22,XM=A2,YP=A3,YM=23; //ID=0x9341
const int TS_LEFT=129,TS_RT=905,TS_TOP=943,TS_BOT=89;

TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);
TSPoint tp;

#define MINPRESSURE 200
#define MAXPRESSURE 1000


uint16_t ID;
uint8_t Orientation = 1;    //Landscape - the workbench has been setup to run the display in landscape mode, if portrait required then workscreens table will need to be updated.

// Assign human-readable names to some common 16-bit color values:
#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

#define MODE_NORMAL 0  // in normal mode the the active screen is drawn as per the screen definition
#define MODE_LARGE 1  // in large mode, one set of figures from the screen defintion is displayed
#define MODE_FULLSCREEN 2  // one item of the screen definition is displayed using as much of the screen as possible



#define DIGITAL_BOX 1 // screen item type controlled by input from a digital pin
#define ANALOG_FIG 2  // screen item type controlled by input from an analogue pin
#define PRECISE_FIG 3  // screen item type controlled by input from a high precision object (INA3221)
#define SERIAL_SCROLL 4   // the whole screen is used as a serial display
#define MENU_BOX 5        // the item drawn is a menu item
#define RESISTANCE 6   // screen item type controlled by input from resistance meter
#define CAPACITANCE 7   // screen item type controlled by input from capacitance meter
#define I2CID 8   // screen item type controlled by input from I2C ID detector
#define MILLIVOLT 9   // screen type controlled by input from millivolt meter
#define SCOPE 10   // the whole screen is used as an oscilloscope display
#define PLOT 11   // the whole screen is used to display a plot of one or more measured values

#define ITEMID   0 // item ID
#define ELIGIBLE 1 // eligibility is second column entry
#define ITEMTYPE 2 // item type is third column entry
#define ITEMX    3 // top x coord
#define ITEMY    4 // top y coord
#define ITEMXS   5 // top x coord in large mode
#define ITEMYS   6 // top y coord in large mode
#define ITEMTX1  7 // leftmost touch area - updated by routine
#define ITEMTX2  8 // rightmost touch area - updated by routine
#define ITEMTY1  9 // topmost touch area - updated by routine
#define ITEMTY2 10 // bottommost touch area - updated by routine
#define ITEM_TOUCH 11 // this item has been touched
#define ITEM_TIMER 12 // item timer
#define PARAM_1 13 // item specific parameter
#define MENU_ACTION 13
#define PARAM_2 14 // item specific parameter
#define COLOUR 14 // item specific parameter
#define PARAM_3 15 // item specific parameter
#define PARAM_4 16 // item specific parameter, usually associated text
#define NORMAL_ELIG 17 // item specific parameter, usually associated text
#define LARGE_ELIG 18 // item specific parameter, usually associated text
#define FULL_ELIG 19 // item specific parameter, usually associated text

#define SCREEN_PARMS 20 // number of parameters in workScreens array

/*
 * The definitions for all objects displayed on the screen are held in the workScreens array
 * workScreens is an integer array holding information about the type, position and characteristcis of a sceen object
 * and holds pointers to text in  when text is associated with an object.
 */

/*  Columns for workScreens array are:
 *  itemID  unique ID number for item, may also be array entry
 *  item eligibility can this item be drawn at this time
 *  item type  indicator for tyoe of object to be draw, used to assign correct subroutine  
 *  item x     top left hand corner of area to be drawn in MODE_NORMAL
 *  item y     top left hand corner of area to be drawn in MODE_NORMAL
 *  item xl    top left hand corner of area to be drawn in MODE_LARGE
 *  item yl    top left hand corner of area to be drawn in MODE_LARGE
 *  item tx1   leftmost limit of touch sensitive area 
 *  item tx2   rightmost limit of touch sensitive area
 *  item ty1   topmost area of touch sensitive area 
 *  item ty2   bottom most area of touch sensitive area
 *  item touched set to TRUE if this area has been touched
 *  item timer the length of time that object is updated after in milliseconds, 0 means only with redraw set
 *  Param_1-4  parameters unique to this object, such as input pin, colour, text
 *  Normal screen eligibility
 *  Large screen eligibility
 *  Full screen eligibility
 *  
 *  
 */
 //                        ID elig, type             x    y   xl   yl tx1 tx2 ty1 ty2 tch tim p1     p2     p3 p4 Nrm lge Ful   main screen definition array, one row per screen item 
int workScreens[][20] =  {{1,  1,   DIGITAL_BOX,     0, 280,   0,  50, 0,  0,  0,  0,  0,  0, 38,    0,     0, 0,  1,  0,  0},             //  <<<---digital input on pin 40
                          {2,  1,   DIGITAL_BOX,    60, 280, 110,  50, 0,  0,  0,  0,  0,  0, 39,    0,     0, 1,  1,  0,  0},         //  <<<---digital input on pin 41
                          {3,  1,   DIGITAL_BOX,   120, 280, 220,  50, 0,  0,  0,  0,  0,  0, 40,    0,     0, 2,  1,  0,  0},        //  <<<---digital input on pin 42
                          {4,  1,   DIGITAL_BOX,   180, 280, 330,  50, 0,  0,  0,  0,  0,  0, 41,    0,     0, 3,  1,  0,  0},       //  <<<---digital input on pin 43
                          {5,  1,   DIGITAL_BOX,   240, 280,   0, 220, 0,  0,  0,  0,  0,  0, 42,    0,     0, 4,  1,  0,  0},        //  <<<---digital input on pin 44
                          {6,  1,   DIGITAL_BOX,   300, 280, 110, 220, 0,  0,  0,  0,  0,  0, 43,    0,     0, 5,  1,  0,  0},      //  <<<---digital input on pin 45
                          {7,  1,   DIGITAL_BOX,   360, 280, 220, 220, 0,  0,  0,  0,  0,  0, 44,    0,     0, 6,  1,  0,  0},      //  <<<---digital input on pin 46
                          {8,  1,   DIGITAL_BOX,   420, 280, 330, 220, 0,  0,  0,  0,  0,  0, 45,    0,     0, 7,  1,  0,  0},      //  <<<---digital input on pin 47
                          {9,  1,   ANALOG_FIG,      0,  50,   0,  35, 0,  0,  0,  0,  0,  0, 63,    BLUE,  0, 11, 1,  0,  0},         //  <<<---analog display in blue from input 62, A8
                          {10, 1,   ANALOG_FIG,      0, 108,   0, 170, 0,  0,  0,  0,  0,  0, 62,    RED,   0, 12, 1,  0,  0},       //  <<<---analog display in red from input 63, A9
                          {11, 1,   ANALOG_FIG,      0, 166, 205,  95, 0,  0,  0,  0,  0,  0, 65,    YELLOW,0, 13, 1,  0,  0},   //  <<<---analog display in yellow from input 64, A10
                          {12, 1,   ANALOG_FIG,      0, 224, 205, 240, 0,  0,  0,  0,  0,  0, 64,    GREEN, 0, 14, 1,  0,  0},   //  <<<---analog display in green from input 65, A11
                          {13, 1,   RESISTANCE,      190, 50, 0, 50, 0,  0,  0,  0,  0,  0, 61,    MAGENTA, 0, 20, 1,  0,  0},
                          {14, 1,   PRECISE_FIG,    310,  50,   0,  50, 0,  0,  0,  0,  0,  0,  3,    GREEN, 0, 20, 1,  0,  0},   //  <<<---input from INA3221 in green
                          {15, 1,   PRECISE_FIG,    310, 125, 240,  50, 0,  0,  0,  0,  0,  0,  2,    RED,   0, 20, 1,  0,  0},    //  <<<---input from INA3221 in red
                          {16, 1,   PRECISE_FIG,    310, 200, 180, 176, 0,  0,  0,  0,  0,  0,  1,    YELLOW,0, 20, 1,  0,  0}, //  <<<---input from INA3221 in yellow
                          {17, 0,   SERIAL_SCROLL,   0, 200, 240, 150, 0,  0,  0,  0,  0,  0, 65,    GREEN, 0, 20, 0,  0,  0},
                          {18, 1,   MENU_BOX,        0,   0, 240, 150, 0,  0,  0,  0,  0,  0,  1,    GREEN, 0, 16, 1,  1,  1},   //  <<<---menu box with action 1, colour green and text 16
                          {19, 1,   MENU_BOX,       75,   0, 240, 150, 0,  0,  0,  0,  0,  0,  2,    CYAN,  0, 8,  1,  1,  1},    //  <<<---menu box with action 2, colour cyan and text 8
                          {20, 1,   MENU_BOX,      185,   0, 240, 150, 0,  0,  0,  0,  0,  0,  3,    YELLOW,0, 9,  1,  1,  1}, //  <<<---menu box with action 3, colour yellow and text 9
                          {21, 1,   MENU_BOX,      280,   0, 240, 150, 0,  0,  0,  0,  0,  0,  4,    RED,   0, 10, 1,  1,  1},   //  <<<---menu box with action 4, colour red and text 10
                          {22, 0,   PLOT,            0, 200, 240, 150, 0,  0,  0,  0,  0,  0, 65,    GREEN, 0, 20, 0,  0,  0},    // plot selected measured values
                          {23, 0,   CAPACITANCE,     0, 200, 240, 150, 0,  0,  0,  0,  0,  0, 65,    GREEN, 0, 20, 0,  0,  0},    // capacitance meter
                          {24, 0,   I2CID,           0, 200, 240, 150, 0,  0,  0,  0,  0,  0, 65,    GREEN, 0, 20, 0,  0,  0},    // I2CID scanner
                          {25, 0,   MILLIVOLT,       0, 200, 240, 150, 0,  0,  0,  0,  0,  0, 65,    GREEN, 0, 20, 0,  0,  0},    // millivolt meter entry if implemented
                          {26, 0,   SCOPE,           0, 200, 240, 150, 0,  0,  0,  0,  0,  0, 65,    GREEN, 0, 20, 0,  0,  0}};   // oscilloscope entry
int itemCount = (sizeof(workScreens)/sizeof(workScreens[0][0]))/SCREEN_PARMS;  // variable for number of items in workScreens array

int reDraw; // indicator that screen is to be redrawn, typically after being erased
int workMode = MODE_NORMAL; // type of display mode in use



// touchInfo structure passes touch screen definitions from logic functions
struct touchInfo {
    int xinfo1;
    int xinfo2;
    int yinfo1;
    int yinfo2;
    };

    long oldTimer;
    long newTimer;

    long xpos, ypos;  //screen coordinates of touch
    int itemTouched = 999; // indicator set when item is touched, reset after timeout 
    int itemTouchedPrev = 999; // indicator set when item is touched, reset after timeout 
    long itemTouchTimer;
#define ITEM_TOUCH_TIMEOUT 2000

const String screenText[30] = {"D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "NORMAL", "LARGE", "FULL SCREEN", "A0", "A1", "A2", "A3", "SERIAL", "MENU", "V", "mA", "NULL"} ;      


void drawScreen(int draw) {
  reDraw=draw;
  for (int i=0; i<itemCount; i++) {
    if (workScreens[i][ELIGIBLE]) {
      switch (workScreens[i][ITEMTYPE]) {
        case DIGITAL_BOX:
            drawDigitalBox(i);
            break;
        case ANALOG_FIG:
            drawAnalogFig(i);
            break;
        case PRECISE_FIG:
            drawPreciseFig(i);
            break;
        case SERIAL_SCROLL:
            drawSerial(i);
            break;      
        case MENU_BOX:
            drawMenu(i);
            break; 
        case RESISTANCE:
            drawResistance(i);
            break;    
      }
    }
  }
  reDraw=0;
}


void setup(void) {

    setupDraw();
    setupTouch(); 
    logicEraseScreen();

    analogReference(DEFAULT);
    
    setupAnalog();
    setupDigital();

    setupMenu();
    setupPrecise();
    setupResistance();
    setupSerial();

    drawScreen(1);
    delay(1000);
}


void loop() {

    newTimer= millis();
    if (newTimer > itemTouchTimer) {
      itemTouchedPrev = 999;
      itemTouched = 999;
      }
    tp = ts.getPoint();   //tp.x, tp.y are ADC values

    // if sharing pins, you'll need to fix the directions of the touchscreen pins
    pinMode(XM, OUTPUT);
    pinMode(YP, OUTPUT);
    // we have some minimum pressure we consider 'valid'
    // pressure of 0 means no pressing!
 if (tp.z > MINPRESSURE && tp.z < MAXPRESSURE) {
    
       updateTouch();
     }
    if (newTimer > oldTimer){
      oldTimer = newTimer + 500;
      drawScreen(0);
      }
}
